﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Linq;
using System.Text;
using System.Threading.Tasks;

namespace XamTextEditorDataErrorCase
{
    public class ViewModel : INotifyPropertyChanged, IDataErrorInfo
    {

        private string _editorText;
        public string EditorText
        {
            get { return _editorText; }
            set
            {
                if(_editorText != value)
                {
                    _editorText = value;
                    ValidateEditorText(false);
                    RaisePropertyChanged("EditorText");
                }
            }
        }

        public ViewModel()
        {
            EditorText = "This can't be shorter than Length = 3";
        }

        public void ValidateEditorText(bool raiseNotification)
        {
            if(_editorText.Length < 3)
            {
                string error = "Editor Text Too Short!!";
                this.SetFieldDataError("EditorText", error, raiseNotification);
            }                       
        }

        #region IDataErrorInfo
        private string _dataError;
        private Dictionary<string, string> _dataErrors = new Dictionary<string, string>();
        public string GetFieldDataError(string fieldName)
        {
            string error;
            _dataErrors.TryGetValue(fieldName, out error);

            return error;
        }

        public string GetRecordDataError()
        {
            return _dataError;
        }

        public bool HasDataError()
        {
            return !string.IsNullOrEmpty(_dataError)
                || null != _dataErrors && _dataErrors.Count > 0;
        }


        public void SetFieldDataError(string fieldName, string error, bool raiseNotification)
        {
            if (string.IsNullOrEmpty(error))
            {
                _dataErrors.Remove(fieldName);

                if (0 == _dataErrors.Count)
                    this.SetRecordDataError(null, true);
            }
            else
            {
                _dataErrors[fieldName] = error;

                this.SetRecordDataError("Error", true);
            }

            if (raiseNotification)
                this.RaisePropertyChanged(fieldName);
        }

        public void SetRecordDataError(string error, bool raiseNotification)
        {
            if (_dataError != error)
            {
                _dataError = error;

                if (raiseNotification)
                    this.RaisePropertyChanged("");
            }
        }

        string IDataErrorInfo.Error
        {
            get
            {
                return _dataError;
            }
        }

        string IDataErrorInfo.this[string fieldName]
        {
            get
            {
                return this.GetFieldDataError(fieldName);
            }
        }
        #endregion

        #region INotifyPropertyChanged
        private void RaisePropertyChanged(string propName)
        {
            if (null != _propertyChangedDelegate)
                _propertyChangedDelegate(this, new PropertyChangedEventArgs(propName));
        }

        private PropertyChangedEventHandler _propertyChangedDelegate;

        event PropertyChangedEventHandler INotifyPropertyChanged.PropertyChanged
        {
            add
            {
                _propertyChangedDelegate = (PropertyChangedEventHandler)Delegate.Combine(_propertyChangedDelegate, value);
            }
            remove
            {
                _propertyChangedDelegate = (PropertyChangedEventHandler)Delegate.Remove(_propertyChangedDelegate, value);
            }
        }
        #endregion  
    }
}
